<?php 

			// db.php
			// 
			// Routines for connecting to the gallery database tables 
			//

			// bm_gallery_db_setup
			//
			// This routine is called during activation of the plugin. It checks 
			// if the required database tables are available, and if not, creates them.
			
			function bm_gallery_db_setup()
			{
	 				 global $wpdb,$table_prefix;

					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery';
					 
					 /* Determine the option table name */ 
					 $option_table_name = $table_prefix . 'bm_gallery_options';
					 
					 /* Check if the table exists */ 
					 if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name)
					 {
					 					 /* Create the table */ 
										  $SQL = 'CREATE TABLE ' .  $table_name . ' (';
											$SQL .= "`theme_id` INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,";
											$SQL .= "`theme_active` INTEGER UNSIGNED NOT NULL DEFAULT 0,";											
					 						$SQL .= "`theme_name` VARCHAR(255) NOT NULL DEFAULT '',";
 					 						$SQL .= "`theme_uri` VARCHAR(255) NOT NULL DEFAULT '',";
					 						$SQL .= "`theme_description` VARCHAR(255) NOT NULL DEFAULT '',";
					 						$SQL .= "`theme_options` VARCHAR(255) NOT NULL DEFAULT '',";											
					 						$SQL .= "`theme_author` VARCHAR(255) NOT NULL DEFAULT '',";
					 						$SQL .= "`theme_author_uri` VARCHAR(255) NOT NULL DEFAULT '',";
					 						$SQL .= "`theme_version` VARCHAR(10) NOT NULL DEFAULT '',";
 					 						$SQL .= "`theme_notes` TEXT NOT NULL DEFAULT '',";
					 						$SQL .= "`theme_filename` VARCHAR(255) NOT NULL DEFAULT '',";											
					 						$SQL .= "PRIMARY KEY(`theme_id`))";
					 
					 						/* Run the query */ 
					 						$wpdb->query($SQL);
						}
						
					 /* Check if the table exists */ 
					 if($wpdb->get_var("SHOW TABLES LIKE '$option_table_name'") != $option_table_name)
					 {
					 					 /* Create the table */ 
										  $SQL = 'CREATE TABLE ' .  $option_table_name . ' (';
											$SQL .= "`theme_option_id` INTEGER UNSIGNED NOT NULL AUTO_INCREMENT,";
											$SQL .= "`theme_id` INTEGER UNSIGNED NOT NULL DEFAULT 0,";											
											
											for($lp = 0; $lp < BM_GALLERY_MAXOPTIONS; $lp++)
											  $SQL .= "`theme_option$lp` TINYINT NOT NULL DEFAULT 0,";
											
					 						$SQL .= "PRIMARY KEY(`theme_option_id`),";
											$SQL .= "INDEX `Index_2`(`theme_id`))";
					 
					 						/* Run the query */ 
					 						$wpdb->query($SQL);
						}
						
			} 
			
			// bm_gallery_db_inserttheme
			//
			// Add a theme to the database
						
			function bm_gallery_db_inserttheme($theme)
			{
	 				 global $wpdb,$table_prefix;
					 
					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery';
					 
					 /* Insert a record into the database */ 
					 $SQL  = "INSERT INTO $table_name (theme_name,theme_uri,theme_description,theme_author,theme_author_uri,theme_filename,theme_version) VALUES (";
					 $SQL .= '"' . $wpdb->escape( $theme['Name'] ) . '","'; 
					 $SQL .= $wpdb->escape( $theme['URI'] )  . '","'; 
					 $SQL .= $wpdb->escape( $theme['Description'] ) . '","'; 
					 $SQL .= $wpdb->escape( $theme['Author'] ) . '","';
					 $SQL .= $wpdb->escape( $theme['Author_URI'] ) . '","';
					 $SQL .= $wpdb->escape( $theme['Filename'] ) . '","';					 					  
					 $SQL .= $wpdb->escape( $theme['Version'] ) . '");';
					      
				 	 $wpdb->query($SQL);
					 
					 /* Return the insert ID */ 
					 return $wpdb->insert_id;  				 
			}
			
			function bm_gallery_db_deletetheme($theme_id)
			{
	 				 global $wpdb,$table_prefix;
					 
					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery';
					 
					 /* Quick and painless */ 
					 $SQL = "DELETE FROM $table_name WHERE theme_id=" . intval($theme_id);
				 	 $wpdb->query($SQL);					 
					 
					 /* Also delete the option page records for this theme */ 
					 bm_gallery_db_delete_options($theme_id);					 
					 
			}
			
			// bm_gallery_db_load
			//
			// Load a single row into memory 
			
			function bm_gallery_db_load($id)
			{
	 				 global $wpdb,$table_prefix;
					 
					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery';
					 $SQL = "SELECT * FROM $table_name WHERE theme_id=" . intval($id);
					 return $wpdb->get_row($SQL,ARRAY_A); 
			}
			
			// bm_gallery_db_save_option
			//
			// This routine saves the selected theme options for a particular theme 
			// into the database. (rounded corners, blue, red ..) 
			
			function bm_gallery_db_save_options($theme_id,$options)
			{
	 				 global $wpdb,$table_prefix;

					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery_options';
					 
					 /* We don't trust ourselves */
					 $theme_id = $wpdb->escape($theme_id);
					 // $option   = $wpdb->escape($option);
					 
					 $SQL    = "INSERT INTO $table_name (theme_id";
					 $VALUES = "VALUES ($theme_id";
					 
					 foreach($options as $value)
					 {
					   $SQL .= ",theme_option$value";
						 $VALUES .= ",1";
					 }
					 
					 $SQL    = $SQL . ') ' . $VALUES . ')';
				 	 $wpdb->query($SQL);					 
			}
			
			// bm_gallery_db_active_options
			//
			// This routine calculates the set of options that are actually used by 
			// any theme (thus avoiding that we show options that are irrelevant)
			// Because this is an intensive query on the database ; and it only changes
			// when we safe a theme -- the results are cached by the calling function for re-use.    
			
			function bm_gallery_db_active_options()
			{
			 				 global $table_prefix,$wpdb;
			
							 /* Determine the table name */
					     $table_name_options = $table_prefix . 'bm_gallery_options';
							 $table_name_themes  = $table_prefix . 'bm_gallery';							 
							 
							 $SQL = "SELECT ";
			
			 				 $sep = "";
							 for ($lp = 0; $lp < BM_GALLERY_MAXOPTIONS; $lp++)
							 {
							 		 $SQL .= $sep . "SUM(theme_option$lp) As theme$lp";
									 $sep = ",";
							 }			
							 
							 $SQL .= " FROM $table_name_themes ga,$table_name_options go WHERE ga.theme_id=go.theme_id and ga.theme_active;";
							 
							 // Retrieve the results 
							 $row = $wpdb->get_row($SQL,ARRAY_A);
							 return $row; 
			}
			
			// bm_gallery_db_delete_options
			//
			// This routine deletes all the options for a given theme 
			
			function bm_gallery_db_delete_options($theme_id)
			{
	 				 global $wpdb,$table_prefix;

					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery_options';
					 
					 /* We don't trust ourselves */
					 $theme_id = $wpdb->escape($theme_id);
				 
					 $SQL = "DELETE FROM $table_name WHERE theme_id=$theme_id;";
				 	 $wpdb->query($SQL);					 
			}
			
			// bm_gallery_db_load_options
			//
			// This routine loads all the options for a particular theme into memory
			// so that we can determine to which catagories is belongs (eg. blue, rounded corners) etc. 
			
			function bm_gallery_db_load_options($theme_id) 
			{
	 				 global $wpdb,$table_prefix;

					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery_options';
					 
					 /* We don't trust ourselves */
					 $theme_id = $wpdb->escape($theme_id);
				 
					 $SQL = "SELECT * FROM $table_name WHERE theme_id=$theme_id;";
				 	 
					 // Query the database
					 $results = $wpdb->get_results($SQL,ARRAY_A);
					 
					 // Simplify the results a little 
					 $row = $results[0]; 
					 if ($row) foreach ($row as $field => $value)
					  if (substr($field,0,12)=='theme_option' && $field!= "theme_option_id")
						 $options[] = $value;
		
					 // Return an array with the results 
					 return $options; 					 
			}

			function bm_gallery_db_preparefilter($filter)
			{
					 $SQL = "";
					 // Limit by search options
					 if ($filter) 
					 {
					    $prefix = "WHERE ";
							$where  = ""; 
					 	  foreach ($filter as $key) 
							{
							   $key = intval($key);
							   $where .= $prefix . "theme_option$key>0"; 
								 $prefix = " AND "; 
							}
							if ($where!="")
							 $SQL .= $where;
					 }
					 return $SQL;
			}

		
		  // bm_gallery_db_getselectedthemes
			//
			// This routine takes a filter of catagories (rounded corners, blue, red ... ) and
			// retrieves all matching themes from the database
			//
			// Param: $filter   - array of option ids (numeric) 
			// 				$page			- if more than one page, the page to retrieve (0,1,2,3...)
		
			function bm_gallery_db_getselectedthemes($filter,$page)
			{
	 				 global $wpdb,$table_prefix;
					 global $bm_gallery_theme_options;

					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery';
					 $table_options_name = $table_prefix . 'bm_gallery_options';
					 
					 $SQL = "SELECT theme_id FROM $table_options_name ";
					 $SQL .= bm_gallery_db_preparefilter($filter);
					 
					 // Limit by number of pages 
					 $page = intval($page); 
					 if ($page<=0) $page = 0;

					 // Now slightly rewrite this
					 $SQL = "SELECT * FROM $table_name WHERE theme_active>0 AND theme_id IN ($SQL)";
					 $SQL .= " LIMIT " . ($page*BM_GALLERY_MAXPERPAGE) . "," .BM_GALLERY_MAXPERPAGE;
					 
					 // Query the database
					 return $wpdb->get_results($SQL,ARRAY_A); 					 
			}
			
			// bm_gallery_db_getnumpages
			//
			// Return the number of themes that match the current filter 			

			function bm_gallery_db_getnummatches($filter)
			{
	 				 global $wpdb,$table_prefix;
					 global $bm_gallery_theme_options;

					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery';
					 $table_options_name = $table_prefix . 'bm_gallery_options';
					 
					 $SQL = "SELECT count(*) FROM $table_options_name,$table_name  ";
					 $SQL .= bm_gallery_db_preparefilter($filter);
					 
					 /* Now slightly rewrite this */
					 if (strpos($SQL,"WHERE")>0) 
					  $SQL .= " AND "; 
					 else
					  $SQL .= " WHERE "; 
					  
					 $SQL .= "  theme_active>0 AND $table_name.theme_id=$table_options_name.theme_id";
					 
					 /* Query the database */
					 return $wpdb->get_var($SQL);
			}
			
			// bm_gallery_db_getallactive
			// 
			// Retrieves a listing of all the active themes in the database
			
			function bm_gallery_db_getallactive()
			{
	 				 global $wpdb,$table_prefix;

					 /* Determine the table name */
					 $table_name = $table_prefix . 'bm_gallery';
		
					 /* Build the query */
					 $SQL = "SELECT theme_id FROM $table_name WHERE theme_active>0";

					 /* Return any results */
					 return $wpdb->get_results($SQL,ARRAY_A); 					 			
			}
 						
?>